package com.rjconsultores.ventaboletos.web.gui.controladores.catalogos;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Controller;
import org.zkoss.util.resource.Labels;
import org.zkoss.zhtml.Messagebox;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zk.ui.event.EventListener;
import org.zkoss.zul.Combobox;
import org.zkoss.zul.Comboitem;
import org.zkoss.zul.Paging;

import com.rjconsultores.ventaboletos.entidad.Empresa;
import com.rjconsultores.ventaboletos.entidad.OCDParam;
import com.rjconsultores.ventaboletos.service.OCDParamService;
import com.rjconsultores.ventaboletos.utilerias.UsuarioLogado;
import com.rjconsultores.ventaboletos.web.utilerias.MyGenericForwardComposer;
import com.rjconsultores.ventaboletos.web.utilerias.MyListbox;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.HibernateSearchObject;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.PagedListWrapper;

@Controller("busquedaOCDParamController")
@Scope("prototype")
public class BusquedaOCDParamController  extends MyGenericForwardComposer {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	@Autowired
	private OCDParamService ocdParamService;

	@Autowired
    private transient PagedListWrapper<OCDParam> plwOCDParam;
	private Paging pagingOCDParam;
	
	private MyListbox ocdParamList;
	
	private Combobox cmbEmpresa;
	private List<Empresa> lsEmpresas;
	
	@Override
	public void doAfterCompose(Component comp) throws Exception {		
		
		lsEmpresas = UsuarioLogado.getUsuarioLogado().getEmpresa();
		
		super.doAfterCompose(comp);
		
		ocdParamList.setItemRenderer(new OCDParamListItemRenderer());
		ocdParamList.addEventListener("onDoubleClick", new EventListener() {

			@Override
			public void onEvent(Event event) throws Exception {
				OCDParam param = (OCDParam) ocdParamList.getSelected();
				visualizaParam(param);
			}
		});
		
		actualizaOCDParamList();
	}
	
	private void visualizaParam(OCDParam _param){
		
		if (_param == null)
			return;
		
		Map<String, Object> args = new HashMap<String, Object>();
		if(_param.getIndPenalizacionOCD() == null) {
			_param.setIndPenalizacionOCD(Boolean.TRUE);
			_param.setIndOcdDinheiro(Boolean.TRUE);
		}
		args.put("ocdParamCorrente", _param);
		args.put("ocdParamList", ocdParamList);

		openWindow("/gui/catalogos/editarOCDParam.zul",
				Labels.getLabel("editarOCDParamController.window.title"), args, MODAL);
		
	}
	
	private void actualizaOCDParamList() {
		HibernateSearchObject<OCDParam> paramBusqueda =
				new HibernateSearchObject<OCDParam>(OCDParam.class, pagingOCDParam.getPageSize());

		paramBusqueda.addFilterEqual("activo", true);
		
		Comboitem itemEmpresa = cmbEmpresa.getSelectedItem();		
		if (itemEmpresa != null) {
			Empresa empresa = (Empresa) itemEmpresa.getValue();
			paramBusqueda.addFilterEqual("empresa", empresa);
		} else {
			paramBusqueda.addFilterIn("empresa", lsEmpresas);
		}
		
		paramBusqueda.addSortDesc("ocdparamId");

		plwOCDParam.init(paramBusqueda, ocdParamList, pagingOCDParam);

		if (ocdParamList.getData().length == 0) {
			try {
				Messagebox.show(Labels.getLabel("MSG.ningunRegistro"),
						Labels.getLabel("busquedaOCDParamController.window.title"),
						Messagebox.OK, Messagebox.INFORMATION);
			} catch (InterruptedException ex) {
			}
		}
	}
	
	public List<Empresa> getLsEmpresas() {
		return lsEmpresas;
	}

	public void setLsEmpresas(List<Empresa> _lsEmpresas) {
		lsEmpresas = _lsEmpresas;
	}
	
	public void onClick$btnPesquisa(Event ev) throws InterruptedException {
		actualizaOCDParamList();
	}

	public void onClick$btnRefresh(Event ev) {
		actualizaOCDParamList();
	}

	public void onClick$btnNovo(Event ev) {
		visualizaParam(new OCDParam());
	}	
}
