package com.rjconsultores.ventaboletos.web.gui.controladores.catalogos;

import java.util.Calendar;
import java.util.List;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Controller;
import org.zkoss.util.resource.Labels;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.Executions;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zul.Button;
import org.zkoss.zul.Messagebox;

import com.rjconsultores.ventaboletos.entidad.Curso;
import com.rjconsultores.ventaboletos.service.CursoService;
import com.rjconsultores.ventaboletos.utilerias.RegistroConDependenciaException;
import com.rjconsultores.ventaboletos.utilerias.UsuarioLogado;
import com.rjconsultores.ventaboletos.web.utilerias.MyGenericForwardComposer;
import com.rjconsultores.ventaboletos.web.utilerias.MyListbox;
import com.rjconsultores.ventaboletos.web.utilerias.MyTextbox;

@Controller("editarCursoController")
@Scope("prototype")
public class EditarCursoController  extends MyGenericForwardComposer {

	private static final long serialVersionUID = 1L;
	@Autowired
	private CursoService CursoService;
	private Curso curso;
	private MyListbox cursoList;
	private static Logger log = Logger.getLogger(EditarCursoController.class);
	private MyTextbox txtNome;
	private Button btnApagar;

	public Curso getCurso() {
		return curso;
	}

	public void setCurso(Curso curso) {
		this.curso = curso;
	}

	@Override
	public void doAfterCompose(Component comp) throws Exception {

		super.doAfterCompose(comp);

		curso = (Curso) Executions.getCurrent().getArg().get("curso");
		cursoList = (MyListbox) Executions.getCurrent().getArg().get("cursoList");

		if (curso.getCursoId() == null) {
			btnApagar.setVisible(Boolean.FALSE);
		} 

		txtNome.focus();
	}

	public void onClick$btnSalvar(Event ev) throws InterruptedException {	
		try {
			curso.setActivo(Boolean.TRUE);
			curso.setFecmodif(Calendar.getInstance().getTime());
			curso.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
			curso.setNomcurso(txtNome.getValue());
			
			List<Curso> lsCurso = CursoService.buscar(curso.getNomcurso());

			boolean podeSalvar = false;
			if (lsCurso.isEmpty()) {
				podeSalvar = true;
			} else {
				if (curso.getCursoId() == null) {
					podeSalvar = false;
				} else {
					for (Curso e : lsCurso) {
						if (e.getCursoId().equals(curso.getCursoId())) {
							podeSalvar = true;
						}
					}
				}
			}

			if (podeSalvar) {
				if (curso.getCursoId() == null) {
					CursoService.suscribir(curso);
					cursoList.addItem(curso);
				} else {
					CursoService.actualizacion(curso);
					cursoList.updateItem(curso);
				}
				Messagebox.show(
						Labels.getLabel("editarCursoController.MSG.suscribirOK"),
						Labels.getLabel("editarCursoController.window.title"),
						Messagebox.OK, Messagebox.INFORMATION);

				closeWindow();
			} else {
				Messagebox.show(
						Labels.getLabel("MSG.Registro.Existe"),
						Labels.getLabel("editarCursoController.window.title"),
						Messagebox.OK, Messagebox.EXCLAMATION);
			}

		} catch (Exception ex) {
			log.error("editarCursoController: " + ex);
			Messagebox.show(
					Labels.getLabel("MSG.Error"),
					Labels.getLabel("editarCursoController.window.title"),
					Messagebox.OK, Messagebox.ERROR);
		}
	}

	public void onClick$btnApagar(Event ev) throws InterruptedException {
		try {
			int resp = Messagebox.show(
					Labels.getLabel("editarCursoController.MSG.borrarPergunta"),
					Labels.getLabel("editarCursoController.window.title"),
					Messagebox.YES | Messagebox.NO, Messagebox.QUESTION);

			if (resp == Messagebox.YES) {

				CursoService.borrar(curso);

				Messagebox.show(
						Labels.getLabel("editarCursoController.MSG.borrarOK"),
						Labels.getLabel("editarCursoController.window.title"),
						Messagebox.OK, Messagebox.INFORMATION);

				cursoList.removeItem(curso);

				closeWindow();
			}
		} catch (RegistroConDependenciaException ex) {
			Messagebox.show(ex.getLocalizedMessage(), Labels.getLabel("editarCursoController.window.title"), Messagebox.OK, Messagebox.ERROR);
		}
	}
}
