package com.rjconsultores.ventaboletos.web.gui.controladores.configuracioneccomerciales;

import java.util.HashMap;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Controller;
import org.zkoss.util.resource.Labels;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zk.ui.event.EventListener;
import org.zkoss.zul.Messagebox;
import org.zkoss.zul.Paging;

import com.rjconsultores.ventaboletos.entidad.TipoTarifaPacote;
import com.rjconsultores.ventaboletos.service.TipoTarifaPacoteService;
import com.rjconsultores.ventaboletos.web.utilerias.MyGenericForwardComposer;
import com.rjconsultores.ventaboletos.web.utilerias.MyListbox;
import com.rjconsultores.ventaboletos.web.utilerias.MyTextbox;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.HibernateSearchObject;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.PagedListWrapper;
import com.rjconsultores.ventaboletos.web.utilerias.render.RenderTipoTarifa;

@Controller("busquedaConfiguracionTarifasPacoteController")
@Scope("prototype")
public class BusquedaConfiguracionTarifasPacoteController extends MyGenericForwardComposer {

	private static final long serialVersionUID = 1L;

	@Autowired
	private transient PagedListWrapper<TipoTarifaPacote> plwTipoTarifa;

	@Autowired
	private TipoTarifaPacoteService tipoTarifaPacoteService;

	private MyTextbox txtDescTarifa;
	private Paging pagingTipoTarifa;
	private MyListbox tipoTarifaList;

	@Override
	public void doAfterCompose(Component comp) throws Exception {
		super.doAfterCompose(comp);

		tipoTarifaList.setItemRenderer(new RenderTipoTarifa());
		tipoTarifaList.addEventListener("onDoubleClick", new EventListener() {

			@Override
			public void onEvent(Event event) throws Exception {
				TipoTarifaPacote tarifa = (TipoTarifaPacote) tipoTarifaList.getSelected();
				selecionarTipoTarifa(tarifa);
			}
		});

		refreshListaTipoTarifa();
	}

	private void refreshListaTipoTarifa() {
		HibernateSearchObject<TipoTarifaPacote> tarifaBusqueda =
				new HibernateSearchObject<TipoTarifaPacote>(TipoTarifaPacote.class, pagingTipoTarifa.getPageSize());

		tarifaBusqueda.addFilterLike("desctipotarifa", "%" + txtDescTarifa.getText().trim().concat("%"));
		tarifaBusqueda.addFilterEqual("activo", Boolean.TRUE);
		tarifaBusqueda.addSortAsc("desctipotarifa");

		plwTipoTarifa.init(tarifaBusqueda, tipoTarifaList, pagingTipoTarifa);
	}

	@SuppressWarnings({ "unchecked", "rawtypes" })
	private void selecionarTipoTarifa(TipoTarifaPacote tarifa) {
		if (tarifa == null) {
			return;
		}

		Map args = new HashMap();
		args.put("tarifa", tarifa);
		args.put("tipoTarifaList", tipoTarifaList);

		openWindow("/gui/configuraciones_comerciales/editarTipoTarifaPacote.zul",
				Labels.getLabel("editarConfiguracionPacoteController.window.title"), args, MODAL);
	}

	

	public void onClick$btnAddTarifa(Event ev) throws InterruptedException {
		selecionarTipoTarifa(new TipoTarifaPacote());
	}

	public void onClick$btnPesquisaTarifa(Event ev) throws InterruptedException {
		refreshListaTipoTarifa();
	}

	public void onClick$btnRefreshTarifa(Event ev) throws InterruptedException {
		refreshListaTipoTarifa();
	}

	public void onClick$btnRemoverTarifa(Event ev) throws InterruptedException {

		int resp = Messagebox.show(
				Labels.getLabel("editarConfiguracionPacoteController.tarifa.MSG.borrarPergunta"),
				Labels.getLabel("editarConfiguracionPacoteController.window.title"),
				Messagebox.YES | Messagebox.NO, Messagebox.QUESTION);

		if (resp == Messagebox.YES) {

			TipoTarifaPacote tarifa = (TipoTarifaPacote) tipoTarifaList.getSelected();
			tipoTarifaPacoteService.borrar(tarifa);

			Messagebox.show(
					Labels.getLabel("editarConfiguracionPacoteController.tarifa.MSG.borrarOK"),
					Labels.getLabel("editarConfiguracionPacoteController.window.title"),
					Messagebox.OK, Messagebox.INFORMATION);

			tipoTarifaList.removeItem(tarifa);
		}
	}

}
